<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author Ship-Quik SL <info@ship-quik.com>
 * @copyright Since 2022 Ship-Quik SL
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace ShipQuik\PrestaShop;

use libphonenumber\PhoneNumberUtil;
use ShipQuik\Shared\ShipQuik_API;
use ShipQuik\Shared\ShipQuik_Helper;
use ShipQuik\Shared\ShipQuik_ShippingDua;
use ShipQuik\Shared\ShipQuik_ShippingTaxes;
use ShipQuik\Shared\ShipQuik_Supplier;
use ShipQuik\Shared\ShipQuik_Suppliers;

class ShipQuik_Shipments
{
    private static $_instance = null;

    protected $parent;

    protected $order_id;

    protected $boxes;

    protected $config;

    public $data;

    protected $PhoneNumberUtil;

    public function __construct($parent = null, $order_id = 0, $boxes = [])
    {
        $this->parent = $parent;
        $this->order_id = $order_id;
        $this->boxes = $boxes;

        $this->config = $parent->config;

        $this->PhoneNumberUtil = PhoneNumberUtil::getInstance();

        $this->data = $this->getData($this->order_id, $this->boxes);
    }

    private function getData($order_id, $boxes)
    {
        $order = new \Order((int) $order_id);

        $supplierInfo = [];

        $result = ShipQuik_BD::getDatabyOrder($order);

        if (!$result) {
            return [];
        }

        $shipment = [];

        $senderAddress = self::getSenderAddress($this->config);
        $customerAddress = ShipQuik_Suppliers::getNormalizeDestinationPrestaShop(new \Address($order->id_address_delivery));

        $id_customer = $order->id_customer;
        $customer = new \Customer((int) $id_customer);
        $customerAddress['email'] = $customer->email;

        // Shipment Object
        $shipment['shipment']['isKg'] = true;

        if (empty($boxes)) {
            $shipment['shipment']['boxes'] = ShipQuik_ShippingPackage::getBoxes($this->config, $order->getProducts());
        } else {
            $shipment['shipment']['boxes'] = $boxes;
        }

        $shipment['shipment']['originAddress'] = [
            'street' => $senderAddress['Street'],
            'city' => $senderAddress['City'],
            'countryCode' => $senderAddress['CountryCode'],
            'postalCode' => $senderAddress['PostalCode'],
            'floor' => '',
            'door' => '',
        ];

        $shipment['shipment']['destinationAddress'] = [
            'street' => $customerAddress['street'],
            'city' => $customerAddress['city'],
            'countryCode' => $customerAddress['countrycode'],
            'postalCode' => $customerAddress['postcode'],
            'floor' => '',
            'door' => '',
        ];

        $senderPhoneFormated = $this->PhoneNumberUtil->parse($this->config->getValueByKey('senderAddressPhoneNumber'), $senderAddress['CountryCode']);

        $comments = $order->getFirstMessage();
        if (!$comments) {
            $comments = '';
        }

        $contact = $customerAddress['contact'];

        $pickupData = json_decode($result['pickup_datetime'], true);

        // Pickup object
        $shipment['pickup'] = [
            'name' => $this->config->getValueByKey('senderAddressCompany'),
            'cif' => $this->config->getValueByKey('senderAddressCif'),
            'phone' => [
                'countryCallingCode' => (string) $senderPhoneFormated->getCountryCode(),
                'number' => $senderPhoneFormated->getNationalNumber(),
                'numberIntl' => '+' . $senderPhoneFormated->getCountryCode() . ' ' . $senderPhoneFormated->getNationalNumber(),
            ],
            'email' => $this->config->getValueByKey('senderAddressEmail'),
            'comments' => $this->config->getValueByKey('comments'),
            'attention' => '',
            'date' => (isset($pickupData['pickup_date']) ? $pickupData['pickup_date'] : ''),
            'schedule' => (isset($pickupData['pickup_time']) ? $pickupData['pickup_time'] : '1'),
        ];

        $name = ($customerAddress['company'] ? $customerAddress['company'] : $customerAddress['contact']);
        $cif = '';

        $phone = [];

        // Delivery object
        if (isset($customerAddress['phone']) && $customerAddress['phone']) {
            $customerPhoneFormated = $this->PhoneNumberUtil->parse($customerAddress['phone'], $customerAddress['countrycode']);

            $shipment['delivery'] = [
                'name' => $name,
                'cif' => $cif,
                'phone' => [
                    'countryCallingCode' => (string) $customerPhoneFormated->getCountryCode(),
                    'number' => $customerPhoneFormated->getNationalNumber(),
                    'numberIntl' => '+' . $customerPhoneFormated->getCountryCode() . ' ' . $customerPhoneFormated->getNationalNumber(),
                ],
                'email' => $customerAddress['email'],
                'comments' => $comments,
                'attention' => $contact,
            ];
        } else {
            $shipment['delivery'] = [
                'name' => $name,
                'cif' => $cif,
                'phone' => [
                    'countryCallingCode' => (string) $senderPhoneFormated->getCountryCode(),
                    'number' => $senderPhoneFormated->getNationalNumber(),
                    'numberIntl' => '+' . $senderPhoneFormated->getCountryCode() . ' ' . $senderPhoneFormated->getNationalNumber(),
                ],
                'email' => $customerAddress['email'],
                'comments' => $comments,
                'attention' => $contact,
            ];
        }

        $supplierInfoDB = $result['supplier_data'];

        $supplierInfo = new ShipQuik_Supplier($this->parent, $shipment['shipment']['boxes'], new \Address($order->id_address_delivery));

        $suppliers = [
            'name' => $supplierInfoDB['supplier'],
            'services' => [$supplierInfoDB['service']],
        ];
        $supplierInfo = $supplierInfo->send($suppliers, true);

        $supplierInfo = (isset($supplierInfo[0]) ? $supplierInfo[0] : []);

        if (empty($supplierInfo)) {
            return [];
        }

        // Supplier object
        $shipment['supplier'] = $supplierInfo;
        $shipment['supplier']['currency'] = '';
        $shipment['supplier']['serviceCode'] = (isset($supplierInfo['serviceCode']) ? $supplierInfo['serviceCode'] : '');

        $ivaPercent = 0;
        $clientType = $this->config->getClientType();

        if ($clientType == 0) {
            $resultTaxes = $this->getTaxesInfo($customerAddress, $senderAddress);

            if ($resultTaxes) {
                $ivaInfo = $resultTaxes['ivaInfo']['percentage'];
            }

            if ($resultTaxes) {
                $ivaInfo = $resultTaxes['ivaInfo']['percentage'];
                $shipment['iva'] = [
                    'ivaPercentage' => $resultTaxes['ivaInfo']['percentage'],
                    'ivaVentaName' => $resultTaxes['ivaInfo']['nameIva'],
                    'ivaCompraName' => 'IGIAND45',
                    'operationVName' => $resultTaxes['operationV'],
                    'operationCName' => $resultTaxes['operationC'],
                ];
            }
        } else {
            $shipment['iva'] = [
                'ivaPercentage' => 0,
                'ivaVentaName' => 'NOSUJETO',
                'ivaCompraName' => 'IGIAND45',
                'operationVName' => 'VNAC',
                'operationCName' => 'SUJPAS',
            ];
        }

        $duaImport = $supplierInfo['dua'];
        $insurance = 0;

        // INSURANCE object
        // pendiente del endpoint y usar Helper::getInsurance()
        $shipment['insurance'] = [
            'insurancePrice' => 0,
            'declaredValue' => 0,
            'insuranceCost' => 0,
            'declaredObjects' => 'Documentos',
            'contentAgreement' => true,
            'isInternational' => false,
            'isTaric' => false,
            'tarics' => [],
        ];

        $shipment['payment'] = [
            'total' => $supplierInfo['salesPrice'] + ($supplierInfo['salesPrice'] * ($ivaPercent / 100)) + $duaImport + $insurance,
            'sale' => $supplierInfo['salesPrice'],
            'method' => (string) $this->config->getPaymentMethod(),
        ];

        $uuid = ShipQuik_Helper::getUUID($supplierInfo['supplier']);

        $shipment['orderId'] = (string) $order->reference;

        $shipment['reference'] = $uuid;
        $shipment['userEmail'] = $this->config->getValueByKey('senderAddressEmail');
        $shipment['clientId'] = $this->config->getIdCliente();

        ShipQuik_BD::setTotalPayment($order, json_encode($shipment, true));

        // echo json_encode($shipment, true);
        // die();

        return $shipment;
    }

    public static function getSenderAddress($config)
    {
        $senderAddress = [];

        $senderAddress['Street'] = $config->getValueByKey('senderAddressStreet');
        $senderAddress['City'] = $config->getValueByKey('senderAddressCity');
        $senderAddress['CountryCode'] = $config->getValueByKey('senderAddressCountryCode');
        $senderAddress['PostalCode'] = $config->getValueByKey('senderAddressPostalCode');
        $senderAddress['Country'] = 'España';

        return $senderAddress;
    }

    public function send()
    {
        $params = [];
        $params['Baseurl'] = $this->config->getValueByKey('endpoints')['createShipment'];
        $params['Data'] = $this->data;

        // echo json_encode($params['Data']); die();

        $curl = new ShipQuik_API($this->parent, $params, true);
        $result = $curl->runCall();

        if ($result == 'renew_token') {
            $this->config->reNewAccessToken();

            $result = $curl->runCall();
        }

        return $result;
    }

    public function getTaxesInfo($customerAddress = [], $senderAddress = [])
    {
        $taxesInfo = [
            'countryCodeOrigin' => $senderAddress['CountryCode'],
            'countryCodeDestination' => $customerAddress['country'],
            'postalCodeOrigin' => $senderAddress['PostalCode'],
            'postalCodeDestination' => $customerAddress['postcode'],
            'idUser' => $this->config->getIdCliente(),
        ];

        $shippingTaxes = new ShipQuik_ShippingTaxes($this->parent, $taxesInfo);
        $resultTaxes = $shippingTaxes->send();

        if ($resultTaxes) {
            return $resultTaxes;
        }

        return [];
    }

    public function getDuaInfoPS($boxes = [], $destination = [], $supplier = '', $declaredValue = 0)
    {
        $duaInfo = new ShipQuik_ShippingDua($this->parent, $boxes, $destination, $supplier, $declaredValue);
        $resultDuaInfo = $duaInfo->send();

        if ($resultDuaInfo) {
            return $resultDuaInfo;
        }

        return [];
    }

    public static function instance()
    {
        if (!isset(self::$_instance)) {
            $class = __CLASS__;
            self::$_instance = new $class();
        }

        return self::$_instance;
    }

    public function __clone()
    {
        trigger_error('La clonación de este objeto no está permitida', E_USER_ERROR);
    }
}
